C***********************************************************************
C*                  Program  ERFLFP v.3.4.Fa                     *
C* *
C*                   Written by Vladimir I. Haltrin                    *
C*               Previous modification: February 24, 1993              *
C                  1992-2007 by Vladimir I. Haltrin                   *
C       Last modification: June 1, 2007 by  Vladimir I. Haltrin        *
C*  *
C* ERFLFP:   E - elastic,  R - Raman,  F - fluorescence,               *
C*           L - light,    F - field,  P - properties.                 *
C*  *
C*   	Calculation of light field parameters in homogeneous sea water *
C*         with taking into account inelastic Raman scattering and     *
C*     	fluorescence by chlorophyll and dissolved yellow substance   *
C***********************************************************************
C  The input files should be in the folder (subdirectory) "Input" on the
C  level of application. The output goes to the folder (subdirectory)
C  "Output" on the same level and you should create it yourself. All old
C  output files will be overridden after running ERFLFP, so remove them
C  from this folder if they should be preserved.
C* *
C    This code gives good results for the depths Z up to 300 m. In order
C    to use it up to 5000 m it is necessary to replace REAL*8 by REAL*10
C    (with the corresponding changes in the FORMATs in the file
C    ERFLFPout.f, i.e.: 1pE14.7E+3 -> 1pE16.7E+4; A14 -> A16) but the
C    time of calculation in this case will increase up to two times.
C    For the depths greater than 5000 m some more significant
C    changes may be necessary (NANs at wavelengths  715 nm).
C***********************************************************************
C     Calculation of light field parameters of a homogeneous ocean:
C     Gordons parameter X=BB/(A+BB), absorption coefficient A, scatte-
C     ring coefficient B, extinction (attenuation) coefficient CE, back- 
C     scattering coefficient BB, mean cosine in the deep regime MU,
C     single scattering albedo AL, diffuse reflectance in the depth RB,
C     diffuse reflectance of the water column RW, diffuse reflectance
C     to direct sun rays striking at angle ZANG to zenith RS,and a depth
C     of the euphotic zone D (the depth of 1% of illumination) as a
C     function of wavelength LAM in nm (or wavenumber NU, 1/cm) and
C     concentrations of chlorophyll pigments C (in mg/m^3), yellow
C     substance Y (dimensionless), hydrosol particles P (in g/m^3), and
C     hydrosol parameter H (dimensionless). Concentrations of small
C     particles PS and large particles PL (both in g/m^3) can be
C     expresed in terms of total particular concentration P and parame-
C     ter H as follows:    PS = P*2*H/(1+H);   PL = P*(1-H)/(1+H).
C     (We use here Optical model of sea water by O.V.Kopelevich:
C     Optical Properties of Ocean Water, Diss. Doct. Phys. and Math.
C     Sci., Institute of Oceanology USSR AS, Moscow, 1981, 442 p.[R1];
C     and theory of V.I.Khalturin: The Self-Consistent Two-Stream
C     Approximation in Radiative Transfer Theory, Izvestiya AN SSSR,
C     Atmos. Ocean Physics, 1985, Vol.21, No.6, p.589-597) [R2];
C     and Vladimir I. Haltrin (= V.I.Khalturin): Propagation of Light
C     in Sea Depth, Ch.2 in Optical Remote Sensing of the Ocean and the
C     Influence of the Atmosphere (V. A.Urdenko and G.Zimmermann, Eds,
C     GDR Acad.of Sci.Publication, BerlinMoscowSevastopol,1985 [R3].
C     The inelastic model coded here corresponds to the paper by
C     Vladimir I. Haltrin and George W. Kattawar, Self-Consistent
C     Solutions to the Equation of Transfer with Elastic and Inelastic
C     Scattering in Oceanic Optics, Part 1: Model, Applied Optics, 1993
C*
C   Wavenumbers NU (in 1/cm) are related to wavelengths LAM (in nm) by
C   the following equation: NU = 10^7/(2**LAM) =1591549.4309189/LAM.
C***********************************************************************
      Program ERFLFP
C***********************************************************************
      IMPLICIT      none
      REAL*8       CONZQ(6)                     !  C, Y, P, H, ZANG, Q
      INTEGER*4    FLAGS(2)                     !  TEST, FULL
C***********************************************************************
      EXTERNAL	INPUTZ,INPUTNLL,INPUTNRR,INPUTRAMAN     ! in ERFLFPin.f
      EXTERNAL	INPUTFLUOR,INPUTLFP,CHECKTEST           ! in ERFLFPin.f
      EXTERNAL	LFPSUB,FCYINIT,SELFCONS,SELFKONS,EUPHOT ! in SELF.f
      EXTERNAL	ELASTIC,RAMANIRR,FLUORIRR               ! in INCOR.f
      EXTERNAL	RAMANDRCOR,FLUORDRCOR                   ! in INCOR.f
      EXTERNAL	CWRITE,INCYBAND,ZWRITE,DRAWDIVLINE      !  in AUXIL.f
      EXTERNAL	OutputLFP,OutLFP_ZL,OutLFP_LZ           ! in ERFLFPout.f
      EXTERNAL	OutpNuLFP,OutLFPnZL,FIG23NU             ! in ERFLFPout.f
      EXTERNAL	R10EXP                                  ! in XYSB.f
C***********************************************************************
C     Input    variables:
C     ------------------ 
      REAL*8       C,Y,P,H,BTR
      INTEGER*4    NR,NDR,NZ,NNL,NNR,NLL(10),NRR(10)
      REAL*8       NU(165),LF(165),Awo(165),Aco(165),LO(165)
      REAL*8       FR(15),Z(31)
      REAL*8       ETC,KLC,LAC,SIGLC,ETY,KLY,LAY,SIGLY

C     Output   variables:
C     ------------------
      REAL*8       A(165),B(165),BB(165),CE(165),AL(165),CW(165)
      REAL*8       X(165),MU(165),RB(165),RO(165),RW(165),RS(165)
      REAL*8       KB(165),KO(165),ALFE(165),AI(165),RI(165)
      REAL*8       ROI(165),RSI(165),ALFI(165),KBI(165),KOI(165)
      REAL*8       KBA(165),KOA(165),KBIA(165),KOIA(165), MUS,PS,PL
      REAL*8       CI(165),ALI(165),XI(165),MUI(165),RWI(165),RBI(165)
      REAL*8       D(109),DI(109),DRR(109),DRC(109),DRY(109),DRF(109)

      REAL*8       EQD(109,31),ESD(109,31),EED(109,31),ERD(109,31)
      REAL*8       NRD(109,31),NRU(109,31),NFD(109,31),NFU(109,31)
      REAL*8       ECD(109,31),EYD(109,31),EFD(109,31),EID(109,31)
      REAL*8       ESU(109,31),ERU(109,31),EKB(109,31)
      REAL*8       EKI(109,31),ECU(109,31),EYU(109,31),EFU(109,31)
      REAL*8       EIU(109,31),ESID(109,31),ESIU(109,31),EEID(109,31)
      REAL*8       KEED(109,31),KEEU(109,31),KEID(109,31),KEIU(109,31)
      REAL*8       EQID(109,31),REZ(109,31),RIZ(109,31),REIZ(109,31)

C     Auxilary variables:
C     ------------------
      LOGICAL    FULLC,CFBAND,YFBAND,GOFC,GOFY
      INTEGER*4  J,K,L,M,N
      REAL*8     XX,LM,KOJ,ROJ,ROL,RBJ,RBL,R10EXP
      REAL*8     BL,CT,BBL,ALFA,ALFIN,ETL,MM,MRS,QS,LAJ
      REAL*8     HZ,PH,KBJ,RSJ,XQD,XSD,XED,XSU,dXQD,dXSD,dXED,dXSU
      REAL*8     XCU,XYU,XFU,XCD,XYD,XFD,XRD,XRU,XL,MUL,MUJ,AB,AT
      REAL*8     dXCU,dXYU,dXFU,dXCD,dXYD,dXFD,dXRD,dXRU,EX,EXI
      REAL*8     Q,K12,K21,K1EX,NVJ,NUJ,DRRJ,DRCJ,DRYJ,DRFJ,AFC,AFY,ZZ
      REAL*8     ZU,ZD,NV(165),ET(165),TK(165),TN(165),AC,AY,BRS,BRE
      REAL*8     BR(165),BC(165),BY(165),ZANG,FC,FY
      REAL*8     PI /3.14159265358979324/,KLN/1.59155D+6/ 

      CHARACTER*9  DATESTRING /"Not impl."/
C
C                              Input:
C
      OPEN (11, FILE = './Input/ERF.in', STATUS = 'OLD')
         READ (11,*) (CONZQ(J), J=1,6)            !  C,Y,P,H, ZANG,Q
         READ (11,*) (FLAGS(J), J=1,2)            !  TEST, FULL
      CLOSE(11)
          C = CONZQ(1)           !  Concentration of chlorophyll a
          Y = CONZQ(2)           !  Concentration of yellow sybstance
          P = CONZQ(3)           !  Total concentration of particles
          H = CONZQ(4)           !  Hydrosol parameter
         PH = P/(1.+H)  
         PL = (1.-H)*PH          !  Concentration of large particles
         PS = 2.*H*PH            !  Concentration of small particles
       ZANG = CONZQ(5)           !  Solar zenith angle 
          Q = CONZQ(6)           !  % coefficient (Q=100. or Q=1.)
C
      CALL INPUTLFP(165, NU,LF,Awo,Aco,LO)
      CALL INPUTRAMAN(BTR,NR,NDR,FR)
      CALL INPUTFLUOR(ETC,KLC,LAC,SIGLC, ETY,KLY,LAY,SIGLY)
      CALL INPUTZ(NZ,Z)
      CALL INPUTNLL(NNL,NLL)
      CALL INPUTNRR(NNR,NRR)
C
      CALL CHECKTEST(FLAGS(1), ETC,ETY,BTR,NR,NDR,FR,LO) !  Enable test
      CALL FCYINIT(ETC,ETY, BC,BY)         !  initialize arrays BC & BY
C
      IF (FLAGS(2).EQ.1) THEN    ! If FLAGS(2) = FULL = 1 then
         FULLC = .TRUE.          ! implement a full calculation
      ELSE                       ! of inelastic components
         FULLC = .FALSE.         ! else do not calculate them
      ENDIF                      ! at certain unimportant wavelengths
C
C      Calculation of sun rays cosine just beneath the sea surface:
C
C     ZANG is a sun zenith angle (in degrees);   (1/MUS)max = 1.5023                                       
      MUS  = SIN(PI*ZANG/180.)/1.34         !  n = 1.34 is a refractive
      MUS = SQRT(1.0-MUS*MUS)               !           index of water
C     MUS is a cosine of direction of sun rays just beneath the surface

C
C        Calculation of basic (elastic) hydrooptical properties:
C
      DO J = 1, 165
         L = J
         CALL LFPSUB(LF(L),Awo(L),Aco(L),C,Y,P,H
     &     ,A(L),B(L),BB(L),CE(L),ALFE(L),AC,AY,BRS,BRE,CW(L),AL(L),XX)
         BR(L) = BTR*BRS      !  Raman scattering coefficient
         BC(L) = AC*BC(L)     !  Chlorophyll fluorescent coefficient
         BY(L) = AY*BY(L)     !  Yellow substance fluor. coefficient
c        BREM(L) = BRE        !  Raman emittance coefficient (not used)
C            A  is a total absorption coefficient
C            AC is a chlorophyll absorption coefficient
C            AY is a yellow substance absorption coefficient
C            B  is a total scattering coefficient
C            BRS  is a (400/LAM)^4
C            BB is a total backscattering coefficient
C            CE is a total extinction coefficient
C            ALFE is a attenuation coefficient Alfa (epsilon)(elastic)
C            CW is a pure water extinction coefficient
C            AL (omega) is a single scattering albedo
           X(L)  = XX                !  X is a Gordons coefficient
         CALL SELFCONS(XX, MUL,RB(L),RO(L),RW(L))
            MU(L)=MUL
         CALL SELFKONS(XX,MUL,MUS, KB(L),KO(L),RS(L))
      ENDDO

C
C       Calculation of elastic irradiances EQD, ESD, EED,ESU,
C   irradiance attenuation coefficients KEED and KEEU, and depth
C   dependent Diffuse Reflectance REZ in visible band of spectrum.
C
      DO L = 1,109
         J = L
         ALFA = ALFE(J)
         KBJ = KB(J)
         MUJ = MU(J)                                    !  Mean cosine
         RSJ = RS(J)
         NUJ = ALFA*KBJ
         LM  = LO(J)
         DO N = 1, NZ
            K = N
            CALL ELASTIC(ALFA,KBJ,MUJ,RSJ,MUS,LM,Z(K)
     &                              , XQD,XSD,XSU, dXQD,dXSD,dXSU, EX)
            EKB(J,K) = EX                          !  EX = exp(-nu*z)
                 XED = XQD+XSD
            EQD(J,K) = EX*XQD
            ESD(J,K) = EX*XSD
            EED(J,K) = EX*XED    !  Downward elastic irradiance 
            REZ(J,K) = XSU/XED   !  Depth dependent elastic DR
            ESU(J,K) = EX*XSU    !  Upward scattered elastic irradiance
                  dXED = dXQD+dXSD
C                             Downward elastic attenuation coefficient:
           KEED(J,K) = NUJ+(dXED-NUJ*XED)/XED
C                             Upward elastic attenuation coefficient:
           KEEU(J,K) = NUJ+(dXSU-NUJ*XSU)/XSU
         ENDDO
      ENDDO

C
C      Calculation of elastic hydrooptical properties (= elastic with
C      corrections to energy losses due to Raman scattering, but without
C      corrections due to inelastic Raman and fluorescence emissions
C      at higher wavelenghths) and auxilary arrays NV,ET,TK and TN:
C
      DO  J = 1, 165
             L   = J
            AT   = A(L)+BR(L)   !  BR is the only one correction term
            BL   = B(L)
         AI(L)   = AT           !  Elasic absorption coefficient
            BBL  = BB(L)        !  Elastic backscattering coefficient
            CT   = AT+BL
         CI(L)   = CT           !  Elasic extinction coefficient
         ALI(L)  = BL/CT        !  Elasic single-scattering albedo
              AB = AT+BBL
            ALFA = AB+BBL 
         ALFI(L) = ALFA         !  Elasic attenuation coefficient
              XL = BBL/AB
          XI(L)  = XL           !  Elasic Gordons parameter
             CALL SELFCONS(XL, MUL,RBI(L),ROI(L),RWI(L))
          MUI(L) = MUL
             CALL SELFKONS(XL,MUL,MUS, KBJ,KOI(L),RSJ)
         KBI(L) = KBJ
         RSI(L) = RSJ
         NV(L) = ALFA*KBJ
           ETL = ALFA/MUS
         ET(L) = ETL
           MM = MUL*MUL
           MRS = MUS*RSJ
           QS = .5+MUL/(1.+MM)
         TK(L) = 2.*MRS *QS*ETL*(MUS*(4.-MM)-MUL)
         TN(L) = MRS*(2.+MUL)
      ENDDO

C
C      Calculation of inelastic upward and downward irradiances
C      EQID, ESID, ERD, ECD, EYD, EID, ESIU, ERU, ECU, EYU, EFU
C      and irradiance attenuation coefficients KEID and KEIU.
C
C   Calls to soubroutines RAMANIRR FLUORIRR, RAMANDRCOR, FLUORDRCOR
C   and ELASTIC to calculate elastic irradiances EQD,ESD,ESU and their
C   derivatives, Raman and fluorescence perturbation terms to Diffuse
C   Reflectance DRR, DRC and DRY, inelastic corrections to irradiances
C   ERD, ERU,ECD, ECU, EYD, EYU and their derivatives -dE**/dz due to
C   Raman scattering, chlorophyll (C) and yellow substance (Y) fluores-
C   cence, and also total irradiances and attenuation coefficients KEID
C   and KEIU, which take into account Raman scattering and fluorescence.
C
      DO L = 1, 109    !  <-- Entry to visible spectrum wavelength loop
         J = L
C        -------------------
         CALL INCYBAND(J, CFBAND,YFBAND)
         GOFC = FULLC.OR.CFBAND
         GOFY = FULLC.OR.YFBAND
C        -------------------
          RBJ = RBI(J)
          ROJ = ROI(J)
          RSJ = RSI(J)
          RBL = 1.+RBJ
          ROL = 1.-ROJ*RBJ
          K12 = .5*RBL/ROL
          KOJ = KOI(J)
          KBJ = KBI(J)
          ALFA = ALFI(J)
          MUJ  = MUI(J)                     !  Mean cosine
          LM = LO(J)
          NVJ = NV(J)
          NUJ = NU(J)
          LAJ = KLN/NUJ           
           FC = (LAJ-LAC)/SIGLC
           FY = (LAJ-LAY)/SIGLY
          AFC = KLC*R10EXP(-.5*FC*FC)  !  Chloroph. fluoresc. exc. band
          AFY = KLY*R10EXP(-.5*FY*FY)  !  Yell. subst. fluor. exc. band

          CALL RAMANDRCOR(BR,FR,NR,NDR, MUS,LO, NV,ET,TK,TN
     &                                           ,ALFA,RBJ,KOJ,J, DRRJ)

          IF (GOFC) THEN
             CALL FLUORDRCOR(BC,MUS,LO,NV,ET,TK,TN,ALFA,RBJ,KOJ,J, DRCJ)          
             DRCJ = DRCJ*AFC
          ELSE
             DRCJ = 0.
          ENDIF

          IF (GOFY) THEN
             CALL FLUORDRCOR(BY,MUS,LO,NV,ET,TK,TN,ALFA,RBJ,KOJ,J, DRYJ)
             DRYJ = DRYJ*AFY
          ELSE
             DRYJ = 0.
          ENDIF

          DRFJ = DRCJ+DRYJ
          DRR(J) = DRRJ
          DRC(J) = DRCJ
          DRY(J) = DRYJ
          DRF(J) = DRFJ
          RI(J) = RSI(J)+DRRJ+DRFJ  !  RI is an Inelastic DR of the sea
C         ============              !  under direct sun illumination
          DO M = 1, NZ              !  <-- Entry to the sea depth loop
             K = M
             ZZ = Z(K)
             CALL ELASTIC(ALFA,KBJ,MUJ,RSJ,MUS,LM, ZZ
     &                             , XQD,XSD,XSU,dXQD,dXSD,dXSU,EXI)            
                 EKI(J,K) = EXI        !  EXI = exp(-nui*z)
                      XED = XQD+XSD
				EQID(J,K) = EXI*XQD
                ESID(J,K) = EXI*XSD
                EEID(J,K) = EXI*XED    !  Downward elastic irradiance
                RIZ(J,K)  = XSU/XED    !  Depth dependent elastic DR
                               !  Upward scattered elastic irradiance:
                ESIU(J,K) = EXI*XSU
C                    XEU = XSU because XQU = 0; for that reason EEIU
                dXED = dXQD+dXSD             !  does not appear here
C               -----------------------
                K1EX = K12*EXI
                HZ = .5*ZZ
             CALL RAMANIRR(BR,FR,NR,NDR, MUS,LO,HZ, NV,ET,TK,TN
     &                       ,ALFA,ROJ,RBJ,KOJ, J, XRD,XRU,dXRD,dXRU)
C            ---------------------------
             IF (GOFC) THEN
                CALL FLUORIRR(BC,MUS,LO,HZ,NV,ET,TK,TN
     &                       ,ALFA,ROJ,RBJ,KOJ, J, XCD,XCU,dXCD,dXCU)
                 XCD =  XCD*AFC
                 XCU =  XCU*AFC
                dXCD = dXCD*AFC
                dXCU = dXCU*AFC
             ELSE
                 XCD = 0.
                 XCU = 0.
                dXCD = 0.
                dXCU = 0.
             ENDIF
C            ---------------------------
             IF (GOFY) THEN
                CALL FLUORIRR(BY,MUS,LO,HZ, NV,ET,TK,TN
     &                       ,ALFA,ROJ,RBJ,KOJ, J,  XYD,XYU,dXYD,dXYU)
                 XYD =  XYD*AFY
                 XYU =  XYU*AFY
                dXYD = dXYD*AFY
                dXYU = dXYU*AFY
             ELSE
                 XYD = 0.
                 XYU = 0.
                dXYD = 0.
                dXYU = 0.
             ENDIF
C            --------------------
                   XFU = XCU+XYU
                   XFD = XCD+XYD
              ECU(J,K) = K1EX*XCU
              EYU(J,K) = K1EX*XYU
              EFU(J,K) = K1EX*XFU     !  Fluorescent upward irradiance
              ECD(J,K) = K1EX*XCD
              EYD(J,K) = K1EX*XYD
              EFD(J,K) = K1EX*XFD     !  Fluoresc. downward irradiance
                    dXFU = dXCU+dXYU
                    dXFD = dXCD+dXYD
                ERD(J,K) = K1EX*XRD        !  Raman downward irradiance
                ERU(J,K) = K1EX*XRU        !  Raman upward irradiance
C                         Total downward irradiance:
		  EID(J,K) = EXI*XED+K1EX*(XRD+XFD)
C                         Total upward irradiance:
		  EIU(J,K) = EXI*XSU+K1EX*(XRU+XFU)
C            --------------------------------------
             K21 = 2.*ROL/RBL                      !  K21 = 1/K12         
              ZD = K21*XED+XRD+XFD
              ZU = K21*XSU+XRU+XFU
C---------> Downward attenuation coefficient:
            KEID(J,K) = (K21*dXED+dXRD+dXFD)/ZD
C---------> Upward attenuation coefficient:
            KEIU(J,K) = (K21*dXSU+dXRU+dXFU)/ZU  
            REIZ(J,K) = ZU/ZD         !  Depth dependent inelastic DR
C---------> Ratios of downward and upward Raman and fluorescent
C---------> irradiances to the total irradiances:
            NRD(J,K) = Q*XRD/ZD       !  We need them for
            NRU(J,K) = Q*XRU/ZU       !  Figures 2 and 3
            NFD(J,K) = Q*XFD/ZD
            NFU(J,K) = Q*XFU/ZU
         ENDDO                          !  <-- end of sea depth loop
      ENDDO              !  <-- end of visible spectrum wavelength loop

C
C     Normalization of the Diffuse Reflectances by 100. (in %)
C     and the eigenvalues KB, KO by ALFA and KBI, KOI by ALFI.
C
	  DO L = 1, 109                  !  (378.9403  780.1713) nm
		 J = L
		   ALFA = ALFE(J)
		 KBA(J) = ALFA*KB(J)
		 KOA(J) = ALFA*KO(J)
		   ALFIN = ALFI(J)
		 KBIA(J) = ALFIN*KBI(J)
		 KOIA(J) = ALFIN*KOI(J)
      ENDDO
C     -------------------------
      IF (Q.EQ.100.) THEN
	     DO L = 1, 109
		    J = L
		    RB(J)  = Q*RB(J)   !  Elastic  sea Diffuse Reflectance
		    RBI(J) = Q*RBI(J)  !  Elastic sea Diffuse Reflectance
		    RO(J)  = Q*RO(J)
		    ROI(J) = Q*ROI(J)
		    RW(J)  = Q*RW(J)   !  Elastic  sea Diffuse Reflectance (DR)
		    RWI(J) = Q*RWI(J)  !  Elastic sea Diffuse Reflectance (DR)
		    RS(J)  = Q*RS(J)   !  Elastic  sea Diffuse Reflectance (DR)
		    RSI(J) = Q*RSI(J)  !  Elastic sea Diffuse Reflectance (DR)
		    DRR(J) = Q*DRR(J)
		    DRC(J) = Q*DRC(J)
		    DRY(J) = Q*DRY(J)
		    DRF(J) = Q*DRF(J)
		    RI(J)  = Q*RI(J)   !  Inelastic DR of the sea
		    DO M = 1, NZ
		       K = M
		       REZ(J,K)  = Q*REZ(J,K)  !  Elastic   depth dependent DR
		       RIZ(J,K)  = Q*RIZ(J,K)  !  Elastic  depth dependent DR
		       REIZ(J,K) = Q*REIZ(J,K) !  Inlastic  depth dependent DR
		    ENDDO
         ENDDO                  !  sea under direct sun illumination
      ENDIF

C
C   Calculation of elastic and inelastic euphotic zone depths D and DI 
C
      DO J  = 1, 109        !  D(J) is 'an elastic' euphotic zone depth
           CALL EUPHOT( MU(J), A(J), D(J) )
           CALL EUPHOT(MUI(J),AI(J), DI(J))
      ENDDO         !  DI(J) is a final 'inelastic' euphotic zone depth

C
C                             Output:
C
      CALL OutputLFP(C,Y,P,H,PL,PS,ZANG,MUS,Q,BTR,ETC,ETY
     &     ,DATESTRING, LF, A,B,CE,BB,AL,D,DI, X,MU,RB,RW,RS,KBA,KOA
     &     ,AI,CI,ALI,XI,MUI,RBI,RSI, RWI,ALFI,DRR,DRF,RI,KBIA,KOIA)

      CALL OutLFP_ZL(C,Y,P,H,PL,PS,ZANG,MUS,Q,BTR,ETC,ETY,NNL,NLL
     &       ,NZ,DATESTRING,Awo,CW,LF, Z,EQD,ESD,EED,ESU,KEED,KEEU,REZ
     &,ERD,ERU,EFD,EFU,KEID,KEIU,RIZ, EQID,ESID,EEID,ESIU,EID,EIU,REIZ)

      CALL OutLFP_LZ(C,Y,P,H,PL,PS,ZANG,MUS,Q,BTR,ETC,ETY,NNL,NLL
     &       ,NZ,DATESTRING,Awo,CW,LF, Z,EQD,ESD,EED,ESU,KEED,KEEU,REZ
     &,ERD,ERU,EFD,EFU,KEID,KEIU,RIZ, EQID,ESID,EEID,ESIU,EID,EIU,REIZ)

      CALL OutLFPnZL(C,Y,P,H,PL,PS,ZANG,MUS,Q,BTR,ETC,ETY,NNR,NRR
     &       ,NZ,DATESTRING,Awo,CW,LF, Z,EQD,ESD,EED,ESU,KEED,KEEU,REZ
     &,ERD,ERU,EFD,EFU,KEID,KEIU,RIZ, EQID,ESID,EEID,ESIU,EID,EIU,REIZ)

      CALL FIG23NU(C,Y,P,H,PL,PS,ZANG,MUS,Q,BTR,ETC,ETY
     &                          ,DATESTRING, NZ,Z, LF,NRD,NRU,NFD,NFU)
     
      END
C***********************************************************************
